package net.sf.itcb.common.portlet.vaadin.exception;

import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.vaadin.ui.Panel;

/**
 * The exception handler mapping stores a map in order to execute a specific mecanism for the exception.
 * The panel is used to define which vaadin main panel is used to attach the mecanism (warning popup...)
 * @author Pierre Le Roux
 */
public class ExceptionHandlerMapping {
	
	private Map<Class, ExceptionHandler> exceptionMapping;
	protected final Logger log = LoggerFactory.getLogger(getClass());
	

	public ExceptionHandlerMapping() {
		this.exceptionMapping = new HashMap<Class, ExceptionHandler>();
	}
	
	public void setExceptionClassName(Map<String, ExceptionHandler> exceptionMapping) {
		log.debug("Initializing exception handlers...");
		try {
			for (Map.Entry<String, ExceptionHandler> entry : exceptionMapping.entrySet()) {
				log.debug("{} Specific handler set", entry.getKey());
				this.exceptionMapping.put(Class.forName(entry.getKey()), entry.getValue());
			}
		}
		catch(ClassNotFoundException cnfe) {
			log.error("ExceptionHandlerMapping is not set because a class doesn't exist in classpath.", cnfe);
		}
	}

	public void handleException(Throwable e, Panel panel) {
		if(e != null) {
			ExceptionHandler handler = exceptionMapping.get(e.getClass());
			if (handler == null) {
				for (Class entry : exceptionMapping.keySet()) {
					if( entry.isInstance(e) ) {
						handler = exceptionMapping.get(entry);
					}
				}
			}
			if(handler != null) {
				handler.handleException(e, panel);
			}
			else {
				// If no exception handler for this exception, we only log it
				log.error("An unexpected error occurs : ", e);
			}
			// Else nothing
		}
	}
}
